// Generated by CoffeeScript 1.11.1
(function() {
  var TerraformError, _globalIgnores, _layoutList, _processors, _processorsMap, buildPriorityList, debug, extname, findDefaultLayout, findFirstFile, findNearestLayout, findProcessor, fs, ignore, isEmpty, isIgnored, list, mime, outputPath, outputType, path, resolveMetaData, type, walkData;

  path = require('path');

  fs = require('fs');

  ignore = require('ignore');

  mime = require('mime');

  debug = require('debug')('fedor:worker');

  TerraformError = require('./error').TerraformError;

  _globalIgnores = ignore().add(['_*', '**/.git/', '**/node_modules/', '**/bower_components/']);


  /**
   * This is our processor map for all the helpers.
   *
   * This is our map. This ideally this would be dynamically created by
   * some sort of plugin architecture but lets not let perfect be the
   * enemy of good :)
   */

  exports.processors = _processors = {
    'js': ['coffee', 'js', 'es'],
    'css': ['styl', 'less', 'scss', 'sass'],
    'html': ['ejs', 'jade', 'md', 'html']
  };

  _processorsMap = {};

  for (type in _processors) {
    list = _processors[type];
    list.forEach(function(ext) {
      return _processorsMap[ext] = type;
    });
  }

  extname = exports.extname = function(filepath) {
    return path.extname(filepath).replace(/^\./, '');
  };


  /**
   * Mime Type
   *
   * Returns the mimetype of a specified filepath, resolve with processors types
   */

  exports.mimeType = function(source) {
    var ext, processorType;
    if (source.indexOf('.') === -1) {
      ext = source;
    } else {
      processorType = findProcessor(source);
      ext = processorType ? processorType : path.extname(source);
    }
    return mime.lookup(ext);
  };


  /**
   * Build Priority List
   *
   * Returns a priority list of files to look for on a given request.
   *
   * `css` and `html` are special extensions that will add `less` and `jade`
   * to the priority list (respectively).
   *
   * e.g
   *
   *    buildPriorityList("foobar")
   *      => ["foobar", "foobar.html", "foobar.jade", "foobar.html.jade"]
   *
   *    buildPriorityList("foobar.css")
   *      => ["foobar.css", "foobar.less", "foobar.css.less"]
   *
   *    buildPriorityList("foobar.html")
   *      => ["foobar.html", "foobar.jade", "foobar.html.jade"]
   *
   *    buildPriorityList("foobar.jade")
   *      => ["foobar.jade"]
   *
   *    buildPriorityList("foobar.html.jade.html")
   *      => ["foobar.html.jade.html", "foobar.html.jade.jade", "foobar.html.jade.html.jade"]
   *
   *    buildPriorityList("hello/foobar")
   *      => ["hello/foobar", "hello/foobar.html", "hello/foobar.jade", "hello/foobar.html.jade"]
   */

  buildPriorityList = exports.buildPriorityList = function(filePath) {
    var ext, processor;
    list = [];
    ext = extname(filePath);
    processor = _processors[ext];
    if (processor) {
      processor.forEach(function(p) {
        var regexp;
        regexp = new RegExp(ext + '$');
        return list.push(filePath.replace(regexp, p));
      });
      processor.forEach(function(p) {
        return list.push(filePath + '.' + p);
      });
    } else {
      if (_processorsMap[ext] === 'html') {
        list.push(filePath);
      } else {
        _processors['html'].forEach(function(p) {
          return list.push(filePath + '.' + p);
        });
      }
    }
    return list.map(function(item) {
      return item.replace(/^\/|^\\|\/$/g, '');
    });
  };


  /**
   * Find First File
   *
   * Takes a directory and an array of files. Returns the first file in the list that exists.
   *
   *    findFirstFile("/path/to/public", ["foo.html", "foo.jade", "foo.html.jade"])
   *      => "foo.jade"
   *
   * Returns null if no file is found.
   */

  findFirstFile = exports.findFirstFile = function(root, dirs, arr) {
    var dir, e, filename, first, fullPath, i, j, k, len, len1, ref;
    if (!arr) {
      arr = dirs;
      dirs = [root];
    }
    filename = arr[0];
    first = null;
    for (j = 0, len = dirs.length; j < len; j++) {
      dir = dirs[j];
      if (!dir) {
        continue;
      }
      fullPath = path.resolve(path.dirname(path.resolve(root, dir, filename)));
      try {
        list = fs.readdirSync(fullPath);
      } catch (error) {
        e = error;
        list = [];
      }
      ref = arr.reverse();
      for (k = 0, len1 = ref.length; k < len1; k++) {
        i = ref[k];
        if (list.indexOf(path.basename(i)) !== -1) {
          return path.relative(root, path.resolve(dir, i));
        }
      }
    }
  };

  _layoutList = buildPriorityList('layout.html');


  /**
   * Find Nearest Layout
   *
   * Walks up the tree to find the nearest layout file. Returns relative path to root
   *
   *    findNearestLayout("/path/to/public", "/path/to/public/nested/dir")
   *      => "layout.jade"
   *
   * Returns null if no layout is found.
   */

  findNearestLayout = exports.findNearestLayout = function(rootPath, dir) {
    var layout;
    dir = path.resolve(rootPath, dir || '');
    layout = findFirstFile(dir, _layoutList);
    if (layout) {
      return path.relative(rootPath, path.join(dir, layout));
    }
    if (path.relative(rootPath, dir) !== '') {
      return findNearestLayout(rootPath, path.resolve(dir, '..'));
    } else {
      return null;
    }
  };


  /**
   * Find Default Layout
   *
   * Returns null if outputing json
   */

  findDefaultLayout = exports.findDefaultLayout = function(rootPath, filePath) {
    var arr;
    arr = filePath.split('.');
    if (arr.length > 2 && arr[1] === 'json') {
      return null;
    } else {
      return findNearestLayout(rootPath, path.dirname(filePath));
    }
  };


  /**
   * Is Empty
   *
   * Checks if Object is empty & returns true or false.
   */

  isEmpty = function(obj) {
    var prop;
    for (prop in obj) {
      if (Object.prototype.hasOwnProperty.call(obj, prop)) {
        return false;
      }
    }
    return true;
  };


  /**
   * Walks directory and build the data object.
   *
   * If we call the resolveMetaData on the public dir
   *
   *     public/
   *       |- data.json
   *       |- articles/
   *       |   `- data.json
   *       `- people
   *           `- data.json
   *
   * We get the following...
   *
   *     {
   *       "_data": {...},
   *       "articles": {
   *         "_data": {...}
   *       },
   *       "people": {
   *         "_data": {...}
   *       }
   *     }
   */

  resolveMetaData = exports.resolveMetaData = function(baseDir) {
    var dataPath, dir, e, obj;
    dir = path.resolve(baseDir);
    obj = {};
    if (isIgnored(dir) || !exports.isTemplate(baseDir)) {
      return obj;
    }
    debug('Scan for meta data => %s', dir);
    try {
      list = fs.readdirSync(dir);
    } catch (error) {
      e = error;
      e.source = 'Config';
      e.dest = 'Config';
      e.lineno = -1;
      e.filename = baseDir;
      e.stack = null;
      throw new TerraformError(e);
    }
    dataPath = path.resolve(dir, '_data');
    if (fs.existsSync(dataPath)) {
      try {
        dataPath = require.resolve(dataPath);
        obj._data = require(dataPath);
        delete require.cache[dataPath];
      } catch (error) {
        e = error;
        if (e.code !== 'MODULE_NOT_FOUND') {
          e.source = 'Data';
          e.dest = 'globals';
          e.lineno = -1;
          e.filename = dataPath;
          e.stack = fs.readFileSync(e.filename);
          throw new TerraformError(e);
        }
      }
    }
    obj._contents = [];
    list.forEach(function(file) {
      var d, filePath, stat;
      filePath = path.resolve(dir, file);
      stat = fs.statSync(filePath);
      if (['_', '.'].indexOf(file[0]) === -1) {
        if (stat.isDirectory()) {
          d = resolveMetaData(filePath);
          if (!isEmpty(d)) {
            return obj[file] = d;
          }
        } else {
          return obj._contents.push(outputPath(file));
        }
      }
    });
    if (obj._contents.length === 0) {
      delete obj._contents;
    }
    return obj;
  };


  /**
   * Get Current
   *
   * Returns current object based on the path of source file
   *
   *    getCurrent("foo/bar/baz.jade")
   *      => { path: ["foo", "bar", "baz"], source: "baz" }
   *
   *    getCurrent("index.html")
   *      => { path: ["index"], source: "index" }
   */

  exports.getCurrent = function(sourcePath) {
    var namespace;
    sourcePath = sourcePath.replace(/\\/g, '/');
    namespace = sourcePath.split('.').slice(0, -1).join('.').split('/');
    return {
      source: namespace[namespace.length - 1],
      path: namespace
    };
  };


  /**
   * Find processor name
   *
   * Returns the processor name matches to the givens filepath
   */

  findProcessor = exports.findProcessor = function(filePath) {
    var ext, ref;
    ext = extname(filePath);
    return (ref = _processorsMap[ext]) != null ? ref : null;
  };


  /**
   * Walk Data Tree
   *
   * Recursive function that returns the data object accociated with path.
   *
   *     var globals = {
   *       "public": {
   *         "articles": {
   *           "_data": {
   *             "hello-world": "You Found Me!"
   *           }
   *         }
   *       }
   *     }
   *
   *     walkData(["public", "articles", "hello-world"], globals) => "You Found Me!"
   */

  walkData = exports.walkData = function(tail, obj) {
    var head;
    tail = tail.slice(0);
    head = tail.shift();
    if (obj.hasOwnProperty(head)) {
      return walkData(tail, obj[head]);
    } else if (obj.hasOwnProperty('_data')) {
      if (obj['_data'][head]) {
        return obj['_data'][head];
      } else {
        return null;
      }
    } else {
      return null;
    }
  };


  /**
   * Output Path
   *
   * Returns output path output for given source file.
   *
   * allowAlternateExt: true. May be turned off for one less RegExp execution
   * (and faster performance)
   *
   * eg.
   *     main.es => main.js
   *     /path/foo/style.scss => /path/foo/style.css
   *     foo.jade => foo.html
   *     foo.html.jade => foo.html
   */

  outputPath = exports.outputPath = function(source, allowAlternateExt) {
    var ext, re, ref;
    if (allowAlternateExt == null) {
      allowAlternateExt = true;
    }
    ext = extname(source);
    type = (ref = _processorsMap[ext]) != null ? ref : '';
    if (allowAlternateExt && type !== ext) {
      if ((new RegExp("^.*\\.(\\w{3,4})\\." + ext + "$", 'i')).test(source)) {
        type = RegExp['$1'];
      }
    }
    re = new RegExp("(\\." + type + ")?\\." + ext + "$", 'i');
    return source = source.replace(re, '.' + type);
  };


  /**
   * Output Type
   *
   * Returns output type source file.
   *
   * eg.
   *     foo.jade       => html
   *     foo.html.jade  => html
   */

  outputType = exports.outputType = function(source) {
    var op;
    op = outputPath(source);
    return extname(op);
  };


  /**
   * Should Ignore
   *
   * Checks to see if path should be ignored.
   *
   * eg.
   *     isIgnored('_foo.html')               => true
   *     isIgnored('.git/HEAD')               => true
   *     isIgnored('node_modules/foo/bar.js') => false
   */

  isIgnored = exports.isIgnored = function(filePath) {
    return _globalIgnores.ignores(filePath);
  };


  /**
   * isTemplate
   *
   * Returns true if file is a template file
   *
   * eg.
   *     isTemplate('foo.jade')         => true
   *     isTemplate('foo.md')           => true
   *     isTemplate('foo.html')         => false
   *     isTemplate('foo/bar.jade')     => true
   *     isTemplate('foo/bar.md')       => true
   *     isTemplate('foo/bar.html')     => false
   *     isTemplate('foo.less')         => false
   *     isTemplate('foo.css')          => false
   *     isTemplate('foo.bar.baz.jade') => true
   */

  exports.isTemplate = function(filePath) {
    return findProcessor(filePath) === 'html';
  };


  /**
   * isStylesheet
   *
   * Returns true if file is a pre-processor stylesheet file
   *
   * eg.
   *     isTemplate('foo.less')         => true
   *     isTemplate('foo.md')           => false
   *     isTemplate('foo.css')          => false
   *     isTemplate('foo.bar.baz.less') => true
   */

  exports.isStylesheet = function(filePath) {
    return findProcessor(filePath) === 'css';
  };


  /**
   * isJavaScript(filePath)
   *
   * Returns true if file is a pre-processor javascript file
   *
   * eg.
   *     isJavaScript('foo.coffee')         => true
   *     isJavaScript('foo.md')             => false
   *     isJavaScript('foo.css')            => false
   *     isJavaScript('foo.bar.baz.coffee') => true
   */

  exports.isJavaScript = function(filePath) {
    return findProcessor(filePath) === 'js';
  };


  /**
   * needsBrowserify
   *
   * Returns true if the code uses require, exports or module but doesn't declare them
   */

  exports.needsBrowserify = function(source) {
    return /^[^#\/'"*]*(require) *\(["'][^)]+/m.test(source) && !/\b(function|var|global) +(require)\b|\b(module|require) *=[^=]/.test(source) || /\bimport +.*? from +/.test(source);
  };


  /**
   * Check the target value matches with the specified pattern(s)
   *
   * Returns true is pattern not provided
   */

  exports.checkPattern = function(filePath, pattern) {
    var ig, result;
    if (pattern === null) {
      return true;
    }
    ig = ignore().add(pattern);
    return result = ig.ignores(filePath) === true;
  };

}).call(this);
