// Generated by CoffeeScript 1.11.1
(function() {
  var fs, helpers, path, processors, scope;

  fs = require('fs');

  path = require('path');

  helpers = require('../../helpers');


  /**
   * Build Processor list for markup.
   *
   * same as doing...
   *
   *    var processors = {
   *      "jade" : require("./jade"),
   *      "md"   : require("./md")
   *    }
   */

  processors = {};

  helpers.processors['html'].forEach(function(type) {
    var e;
    try {
      return processors[type] = require("./" + type);
    } catch (error) {
      e = error;
      return console.error("The specific processors not found: " + type + "\n", e);
    }
  });


  /**
   * Scope
   *
   * A closure to properly scope the `partial` function so that
   * we can pass the `partial` function into the template and it
   * will have access to the correct local variables.
   */

  scope = module.exports = function(projectPath, parentLocals) {
    if (!parentLocals) {
      parentLocals = {};
    }
    return {
      partial: function(relPath, partialLocals) {
        var e, ext, fileContents, filePath, layout, local, locals, output, partialCurrent, priorityList, render, templateLocals, tmpl;
        priorityList = helpers.buildPriorityList(relPath);
        relPath = helpers.findFirstFile(projectPath, priorityList);
        if (!relPath) {
          return null;
        }
        filePath = path.resolve(projectPath, relPath);
        partialCurrent = helpers.getCurrent(relPath);
        templateLocals = helpers.walkData(partialCurrent.path, parentLocals["public"]);
        fileContents = fs.readFileSync(filePath);
        ext = path.extname(relPath).replace(/^\./, '');
        if (!fileContents) {
          return null;
        }
        if (!partialLocals) {
          partialLocals = {};
        }
        locals = {};
        for (local in parentLocals) {
          locals[local] = parentLocals[local];
        }
        for (local in templateLocals) {
          if (local !== 'layout') {
            locals[local] = templateLocals[local];
          }
        }
        for (local in partialLocals) {
          locals[local] = partialLocals[local];
        }
        if (locals.hasOwnProperty('layout')) {
          layout = locals['layout'];
        }
        delete locals['layout'];
        locals.partial = scope(path.dirname(filePath), locals).partial;
        tmpl = processors[ext](fileContents, {
          filename: filePath,
          basedir: projectPath
        });
        try {
          render = tmpl.compile();
          output = render(locals);
        } catch (error) {
          e = error;
          if (e.source && e.dest) {
            throw e;
          } else {
            throw tmpl.parseError(e);
          }
        }
        if (layout) {
          output = scope(projectPath, locals).partial(layout, {
            "yield": output
          });
        }
        return output;
      }
    };
  };

}).call(this);
