// Generated by CoffeeScript 1.11.1
(function() {
  var Config, ProcessorFactory, debug, fs, helpers, initWorker, path;

  fs = require('fs');

  path = require('path');

  helpers = require('./helpers');

  debug = require('debug')('fedor:worker');

  Config = require('./config');

  ProcessorFactory = require('./processors');


  /**
   * Initialize a processor engine instance
   *
   * This sets the home base directory for the app which affects
   * where we begin walking to build the home directory.
   *
   */

  initWorker = function(root, options) {
    var data, globals, key, layout, layoutPriorityList, templateObject;
    if (options == null) {
      options = {};
    }
    globals = options.globals;
    layoutPriorityList = helpers.buildPriorityList('layout.html');
    layout = helpers.findFirstFile(root, layoutPriorityList);
    data = helpers.resolveMetaData(root);
    templateObject = {
      "public": data
    };
    for (key in globals) {
      templateObject[key] = globals[key];
    }
    return {
      render: function(filePath, locals, callback) {
        var dirname, e, error, output, processor, processorType, tmpLocals;
        filePath = filePath.replace(/^\\/g, '');
        if (!callback) {
          callback = locals;
          locals = {};
        }
        if (helpers.isIgnored(filePath)) {
          return callback(null, null);
        }
        processorType = helpers.findProcessor(filePath);
        if (!processorType) {
          return callback(null, null);
        }
        processor = ProcessorFactory.get(processorType);
        if (!processor) {
          return callback(null, null);
        }
        debug('Processing by processor [%s], file => %s', processorType, filePath);
        if (processorType === 'html') {
          locals.current = helpers.getCurrent(filePath);
          if (!locals.hasOwnProperty('layout')) {
            tmpLocals = helpers.walkData(locals.current.path, data);
            if (tmpLocals && tmpLocals.hasOwnProperty('layout')) {
              if (tmpLocals['layout'] === false) {
                locals['layout'] = null;
              } else if (tmpLocals['layout'] !== true) {
                dirname = path.dirname(filePath);
                layoutPriorityList = helpers.buildPriorityList(path.join(dirname, tmpLocals['layout'] || ''));
                layoutPriorityList.push(tmpLocals['layout']);
                locals['layout'] = helpers.findFirstFile(root, layoutPriorityList);
              }
            }
            if (!locals.hasOwnProperty('layout')) {
              locals['layout'] = helpers.findDefaultLayout(root, filePath);
            }
          }
          try {
            return output = processor(root, templateObject).partial(filePath, locals);
          } catch (error1) {
            e = error1;
            error = e;
            return output = null;
          } finally {
            callback(error, output);
          }
        } else {
          return processor(filePath, options, callback);
        }
      }
    };
  };

  module.exports = function() {
    Config.init();
    return {
      init: initWorker
    };
  };

}).call(this);
