// Generated by CoffeeScript 1.11.1

/**
 * CSSWorker - A css compiler based on postcss, with plugins config extenssion.
 *
 * @author Allex Wang <http://iallex.com/>
 */

(function() {
  var CSSWorker, checksum, debug, defaultCachebuster, extend, merge, postcss, ref,
    slice = [].slice;

  postcss = require('postcss');

  debug = require('debug')('cssworker');

  ref = require('./'), merge = ref.merge, checksum = ref.checksum;

  extend = function() {
    var args, j, k, len, r, s;
    r = arguments[0], args = 2 <= arguments.length ? slice.call(arguments, 1) : [];
    for (j = 0, len = args.length; j < len; j++) {
      s = args[j];
      for (k in s) {
        if (Object.prototype.hasOwnProperty.call(s, k)) {
          r[k] = s[k];
        }
      }
    }
    return r;
  };

  defaultCachebuster = function(p) {
    return checksum(p);
  };

  CSSWorker = (function() {
    CSSWorker._instance = null;

    CSSWorker.builtinPlugins = ['postcss-assets', 'autoprefixer', 'cssnano'];


    /**
     * Get a singleton CSSWorker instance
     * @singleton
     */

    CSSWorker.create = function(options) {
      var ref1;
      return (ref1 = CSSWorker._instance) != null ? ref1 : (CSSWorker._instance = new CSSWorker(options));
    };

    function CSSWorker(options) {
      var basePath, plugins;
      if (options == null) {
        options = {};
      }
      basePath = options.basePath;
      if (!basePath) {
        throw new Error('basePath not valid.');
      }
      options.env = options.env || process.env.NODE_ENV || 'development';
      plugins = merge(CSSWorker.builtinPlugins.map(function(n) {
        return {
          name: n
        };
      }), options.plugins, {
        pk: 'name'
      });
      this.options = options;
      this.plugins = [];
      this.loadPlugins(plugins);
      this.worker = postcss(this.plugins);
    }

    CSSWorker.prototype.loadPlugins = function(plugins) {
      var defaults, env, j, len, n, plugin, results;
      debug('Try to load plugins => %O', plugins);
      env = this.options.env;
      defaults = {
        'autoprefixer': {
          add: true,
          cascade: true
        },
        'postcss-assets': {
          basePath: this.options.basePath,
          cachebuster: defaultCachebuster
        },
        'cssnano': {
          enable: env !== 'development',
          zindex: false
        }
      };
      results = [];
      for (j = 0, len = plugins.length; j < len; j++) {
        plugin = plugins[j];
        n = plugin;
        results.push((function(_this) {
          return function(n, plugin) {
            var conf, e, loadByName;
            if (typeof plugin === 'object' && plugin.name) {
              n = plugin.name;
              conf = extend({}, defaults[n], plugin.config);
              if (plugin.enable === false || conf.enable === false) {
                return;
              }
            }
            if (typeof n === 'string') {
              loadByName = 1;
              if (conf == null) {
                conf = defaults[n];
              }
            }
            debug("parse %s => %j", n, conf);
            try {
              if (loadByName) {
                n = require(n);
              }
              if (typeof n === 'function' && n.hasOwnProperty('postcss')) {
                n = n(conf);
              }
              return _this.addPlugin(n);
            } catch (error) {
              e = error;
              return console.error("Load plugin %j failed. => %O", plugin, e);
            }
          };
        })(this)(n, plugin));
      }
      return results;
    };

    CSSWorker.prototype.addPlugin = function(plugin) {
      var i, j, len, o, plugins, ref1;
      if (!plugin || !plugin.postcssVersion) {
        return;
      }
      plugins = (ref1 = this.plugins) != null ? ref1 : (this.plugins = []);
      for (i = j = 0, len = plugins.length; j < len; i = ++j) {
        o = plugins[i];
        if (o.postcssPlugin === plugin.postcssPlugin) {
          plugins.splice(i, 1, plugin);
          return;
        }
      }
      return plugins.push(plugin);
    };

    CSSWorker.prototype.process = function(source, options) {
      var filename, postcssOptions, sourcemap;
      filename = options.filename;
      sourcemap = options.sourcemap;
      postcssOptions = {
        from: filename,
        to: filename,
        map: {
          inline: false,
          prev: sourcemap,
          annotation: false
        }
      };
      return this.worker.process(source, postcssOptions);
    };

    return CSSWorker;

  })();

  module.exports = CSSWorker;

}).call(this);
