// Allex Wang for fedor engineDependencies installation

const fs = require('fs');
const path = require('path');
const semver = require('semver')
const pify = require('pify')
const conf = require('../package.json');
const child_process = require('child_process');
const exec = child_process.exec;
const deps = conf.engineDependencies || {};
const mkdirp = require('mkdirp')
const debug = require('debug')('fedor/install');
const npminstall = require('npminstall');
const co = require('co');

const config = require('../lib/config');
const FEDOR_HOME = config.FEDOR_HOME;

const checkModuleVer = (m, rev) => {
  let p, v;
  try {
    p = require.resolve(`${m}/package.json`)
    v = require(p).version
    debug(`Check module: ${m}@${v} <<- ${rev}`)
  } catch (e) {
    return false
  }
  return semver.satisfies(v, rev)
}

const install = () => {
  let pkgs = [];

  config.init();

  Object.keys(deps).forEach(name => {
    if (!checkModuleVer(name, deps[name]))
      pkgs.push({ name: name, version: deps[name] });
  });

  if (pkgs.length > 0) {

    console.log('Installing additional dependencies...')

    debug('packages => %s', pkgs);

    // Create fedor config directories
    pify(mkdirp)(FEDOR_HOME)
      .then(p => FEDOR_HOME)
      .then(dir =>
        co(function* () {
          yield npminstall({
            root: dir,
            pkgs: pkgs,
            registry: 'http://registry.npm.taobao.org/'
          })
        })
      )
      .catch(e => console.error(e))
  }
}

install()

